@@ -2,13 +2,20 @@
 ndk_loc = node[:settings][:ndk_loc]
 user = node[:settings][:user]
 
-execute "add-android-ndk-path" do
-  user user
-  command "echo \"export PATH=\\$PATH:#{ndk_loc} #PATH-NDK\" >> /home/#{user}/.bsenv"
-  not_if "grep PATH-NDK /home/#{user}/.bsenv"
+script "setup-android-ndk" do
+  timeout 14400
+  interpreter "bash"
+  user node[:settings][:user]
+  cwd "/tmp"
+  code "
+    mkdir #{ndk_loc}
+  "
+  not_if do
+    File.exists?("#{ndk_loc}")
+  end
 end
 
-script "setup-android-ndk" do
+script "setup-android-ndk-r9b" do
   timeout 14400
   interpreter "bash"
   user node[:settings][:user]
@@ -21,10 +28,30 @@ script "setup-android-ndk" do
     fi
     tar jxvf /vagrant/cache/android-ndk-r9b-linux-x86$SUFFIX.tar.bz2
     tar jxvf /vagrant/cache/android-ndk-r9b-linux-x86$SUFFIX-legacy-toolchains.tar.bz2
-    mv android-ndk-r9b #{ndk_loc}
+    mv android-ndk-r9b #{ndk_loc}/r9b
   "
   not_if do
-    File.exists?("#{ndk_loc}")
+    File.exists?("#{ndk_loc}/r9b")
+  end
+end
+
+script "setup-android-ndk-r10e" do
+  timeout 14400
+  interpreter "bash"
+  user node[:settings][:user]
+  cwd "/tmp"
+  code "
+    if [ `uname -m` == 'x86_64' ] ; then
+       SUFFIX='_64'
+    else
+       SUFFIX=''
+    fi
+    chmod u+x /vagrant/cache/android-ndk-r10e-linux-x86$SUFFIX.bin
+    /vagrant/cache/android-ndk-r10e-linux-x86$SUFFIX.bin x
+    mv android-ndk-r10e #{ndk_loc}/r10e
+  "
+  not_if do
+    File.exists?("#{ndk_loc}/r10e")
   end
 end
 
