@@ -3,16 +3,20 @@
 # You may want to alter these before running ./makebuildserver
 
 # Name of the base box to use
-basebox = "testing32"
+basebox = "jessie32"
 
 # Location where testing32.box can be found, if you don't already have
 # it. For security reasons, it's recommended that you make your own
 # in a secure environment using trusted media (see the manual) but
 # you can use this default if you like...
-baseboxurl = "https://f-droid.org/testing32.box"
+baseboxurl = "https://f-droid.org/jessie32.box"
 
+# The amount of RAM the build server will have
 memory = 3584
 
+# The number of CPUs the build server will have
+cpus = 1
+
 # Debian package proxy server - if you have one, e.g. "http://192.168.0.19:8000"
 aptproxy = None
 
