@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # checkupdates.py - part of the FDroid server tools
-# Copyright (C) 2010-13, Ciaran Gultnieks, ciaran@ciarang.com
+# Copyright (C) 2010-2015, Ciaran Gultnieks, ciaran@ciarang.com
 # Copyright (C) 2013-2014 Daniel Martí <mvdan@mvdan.cc>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -84,7 +84,7 @@ def check_http(app):
 # Whether this can be used reliably or not depends on
 # the development procedures used by the project's developers. Use it with
 # caution, because it's inappropriate for many projects.
-# Returns (None, "a message") if this didn't work, or (version, vercode) for
+# Returns (None, "a message") if this didn't work, or (version, vercode, tag) for
 # the details of the current version.
 def check_tags(app, pattern):
 
@@ -128,6 +128,7 @@ def check_tags(app, pattern):
 
         if repotype in ('git',):
             tags = vcs.latesttags(tags, 5)
+            logging.debug("Latest tags: " + ','.join(tags))
 
         for tag in tags:
             logging.debug("Check tag: '{0}'".format(tag))
@@ -211,7 +212,8 @@ def check_repomanifest(app, branch=None):
         if not package:
             return (None, "Couldn't find package ID")
         if package != appid:
-            return (None, "Package ID mismatch")
+            return (None, "Package ID mismatch - expected {0}, got {1}"
+                    .format(appid, package))
         if not version:
             return (None, "Couldn't find latest version name")
         if not vercode:
@@ -364,13 +366,6 @@ def fetch_autoname(app, tag):
     else:
         logging.debug("...couldn't get autoname")
 
-    if app['Current Version'].startswith('@string/'):
-        cv = common.version_name(app['Current Version'], app_dir, flavours)
-        if app['Current Version'] != cv:
-            app['Current Version'] = cv
-            if not commitmsg:
-                commitmsg = "Fix CV of {0}".format(common.getappname(app))
-
     return commitmsg
 
 
