@@ -22,57 +22,48 @@ import re
 import logging
 import common
 import metadata
+import sys
 from collections import Counter
+from sets import Set
 
 config = None
 options = None
 
+
+def enforce_https(domain):
+    return (re.compile(r'.*[^sS]://[^/]*' + re.escape(domain) + r'/.*'),
+            domain + " URLs should always use https://")
+
+https_enforcings = [
+    enforce_https('github.com'),
+    enforce_https('gitlab.com'),
+    enforce_https('gitorious.org'),
+    enforce_https('apache.org'),
+    enforce_https('google.com'),
+    enforce_https('svn.code.sf.net'),
+    enforce_https('googlecode.com'),
+]
+
+http_warnings = https_enforcings + [
+    (re.compile(r'.*github\.com/[^/]+/[^/]+\.git'),
+     "Appending .git is not necessary"),
+    # TODO enable in August 2015, when Google Code goes read-only
+    # (re.compile(r'.*://code\.google\.com/.*'),
+    #  "code.google.com will be soon switching down, perhaps the project moved to github.com?"),
+]
+
 regex_warnings = {
-    'Web Site': [
-        (re.compile(r'.*[^sS]://github\.com/.*'),
-         "github URLs should always use https:// not http://"),
-        (re.compile(r'.*[^sS]://code\.google\.com/.*'),
-         "code.google.com URLs should always use https:// not http://"),
+    'Web Site': http_warnings + [
     ],
-    'Source Code': [
-        (re.compile(r'.*[^sS]://github\.com/.*'),
-         "github URLs should always use https:// (not http://, git://, or git@)"),
-        (re.compile(r'.*code\.google\.com/p/[^/]+[/]*$'),
-         "/source is missing"),
-        (re.compile(r'.*[^sS]://code\.google\.com/.*'),
-         "code.google.com URLs should always use https:// not http://"),
-        (re.compile(r'.*[^sS]://dl\.google\.com/.*'),
-         "dl.google.com URLs should always use https:// not http://"),
-        (re.compile(r'.*[^sS]://gitorious\.org/.*'),
-         "gitorious URLs should always use https:// (not http://, git://, or git@)"),
+    'Source Code': http_warnings + [
     ],
-    'Repo': [
-        (re.compile(r'.*[^sS]://code\.google\.com/.*'),
-         "code.google.com URLs should always use https:// not http://"),
-        (re.compile(r'.*[^sS]://dl\.google\.com/.*'),
-         "dl.google.com URLs should always use https:// not http://"),
-        (re.compile(r'.*[^sS]://github\.com/.*'),
-         "github URLs should always use https:// (not http://, git://, or git@)"),
-        (re.compile(r'.*[^sS]://gitorious\.org/.*'),
-         "gitorious URLs should always use https:// (not http://, git://, or git@)"),
-        (re.compile(r'.*[^sS]://[^.]*\.googlecode\.com/svn/?.*'),
-         "Google Code SVN URLs should always use https:// (not http:// or svn://)"),
-        (re.compile(r'.*[^sS]://svn\.apache\.org/repos/?.*'),
-         "Apache SVN URLs should always use https:// (not http:// or svn://)"),
-        (re.compile(r'.*[^sS]://svn\.code\.sf\.net/.*'),
-         "Sourceforge SVN URLs should always use https:// (not http:// or svn://)"),
+    'Repo': https_enforcings + [
     ],
-    'Issue Tracker': [
-        (re.compile(r'.*code\.google\.com/p/[^/]+[/]*$'),
-         "/issues is missing"),
-        (re.compile(r'.*[^sS]://code\.google\.com/.*'),
-         "code.google.com URLs should always use https:// not http://"),
+    'Issue Tracker': http_warnings + [
         (re.compile(r'.*github\.com/[^/]+/[^/]+[/]*$'),
          "/issues is missing"),
-        (re.compile(r'.*[^sS]://github\.com/.*'),
-         "github URLs should always use https:// not http://"),
-        (re.compile(r'.*[^sS]://gitorious\.org/.*'),
-         "gitorious URLs should always use https:// not http://"),
+    ],
+    'Changelog': http_warnings + [
     ],
     'License': [
         (re.compile(r'^(|None|Unknown)$'),
@@ -81,6 +72,12 @@ regex_warnings = {
     'Summary': [
         (re.compile(r'^$'),
          "Summary yet to be filled"),
+        (re.compile(r'.*\b(free software|open source)\b.*', re.IGNORECASE),
+         "No need to specify that the app is Free Software"),
+        (re.compile(r'.*((your|for).*android|android.*(app|device|client|port|version))', re.IGNORECASE),
+         "No need to specify that the app is for Android"),
+        (re.compile(r'.*[a-z0-9][.!?]( |$)'),
+         "Punctuation should be avoided"),
     ],
     'Description': [
         (re.compile(r'^No description available$'),
@@ -94,38 +91,21 @@ regex_warnings = {
     ],
 }
 
-regex_pedantic = {
-    'Web Site': [
-        (re.compile(r'.*github\.com/[^/]+/[^/]+\.git'),
-         "Appending .git is not necessary"),
-        (re.compile(r'.*code\.google\.com/p/[^/]+/[^w]'),
-         "Possible incorrect path appended to google code project site"),
-    ],
-    'Source Code': [
-        (re.compile(r'.*github\.com/[^/]+/[^/]+\.git'),
-         "Appending .git is not necessary"),
-        (re.compile(r'.*code\.google\.com/p/[^/]+/source/.*'),
-         "/source is often enough on its own"),
-    ],
-    'Repo': [
-        (re.compile(r'^http://.*'),
-         "use https:// if available"),
-        (re.compile(r'^svn://.*'),
-         "use https:// if available"),
-    ],
-    'Issue Tracker': [
-        (re.compile(r'.*code\.google\.com/p/[^/]+/issues/.*'),
-         "/issues is often enough on its own"),
-        (re.compile(r'.*github\.com/[^/]+/[^/]+/issues/.*'),
-         "/issues is often enough on its own"),
-    ],
-    'Summary': [
-        (re.compile(r'.*\b(free software|open source)\b.*', re.IGNORECASE),
-         "No need to specify that the app is Free Software"),
-        (re.compile(r'.*[a-z0-9][.,!?][ $]'),
-         "Punctuation should be avoided"),
-    ],
-}
+categories = Set([
+    "Children",
+    "Development",
+    "Games",
+    "Internet",
+    "Multimedia",
+    "Navigation",
+    "Office",
+    "Phone & SMS",
+    "Reading",
+    "Science & Education",
+    "Security",
+    "System",
+    "Wallpaper",
+])
 
 
 def main():
@@ -144,18 +124,12 @@ def main():
         print '    %s' % message
         count['warn'] += 1
 
-    def pwarn(message):
-        if options.pedantic:
-            warn(message)
-
     # Parse command line...
     parser = OptionParser(usage="Usage: %prog [options] [APPID [APPID ...]]")
     parser.add_option("-v", "--verbose", action="store_true", default=False,
                       help="Spew out even more information than normal")
     parser.add_option("-q", "--quiet", action="store_true", default=False,
                       help="Restrict output to warnings and errors")
-    parser.add_option("-p", "--pedantic", action="store_true", default=False,
-                      help="Show pedantic warnings that might give false positives")
     (options, args) = parser.parse_args()
 
     config = common.read_config(options)
@@ -176,19 +150,16 @@ def main():
             if not curbuild or int(build['vercode']) > int(curbuild['vercode']):
                 curbuild = build
 
-        # Potentially incorrect UCM
+        # Incorrect UCM
         if (curbuild and curbuild['commit']
-                and app['Update Check Mode'] == 'RepoManifest' and
-                any(s in curbuild['commit'] for s in '.,_-/')):
-            pwarn("Last used commit '%s' looks like a tag, but Update Check Mode is '%s'" % (
+                and app['Update Check Mode'] == 'RepoManifest'
+                and not curbuild['commit'].startswith('unknown')
+                and curbuild['vercode'] == app['Current Version Code']
+                and not curbuild['forcevercode']
+                and any(s in curbuild['commit'] for s in '.,_-/')):
+            warn("Last used commit '%s' looks like a tag, but Update Check Mode is '%s'" % (
                 curbuild['commit'], app['Update Check Mode']))
 
-        # Dangerous auto updates
-        if curbuild and app['Auto Update Mode'] != 'None':
-            for flag in ['target', 'srclibs', 'scanignore']:
-                if curbuild[flag]:
-                    pwarn("Auto Update Mode is enabled but '%s' is manually set at '%s'" % (flag, curbuild[flag]))
-
         # Summary size limit
         summ_chars = len(app['Summary'])
         if summ_chars > config['char_limits']['Summary']:
@@ -199,22 +170,36 @@ def main():
         if app['Web Site'] and app['Source Code']:
             if app['Web Site'].lower() == app['Source Code'].lower():
                 warn("Website '%s' is just the app's source code link" % app['Web Site'])
-                app['Web Site'] = ''
+
+        # Missing or incorrect categories
+        if not app['Categories']:
+            warn("Categories are not set")
+        for categ in app['Categories']:
+            if categ not in categories:
+                warn("Category '%s' is not valid" % categ)
+
+        if app['Name'] and app['Name'] == app['Auto Name']:
+            warn("Name '%s' is just the auto name" % app['Name'])
 
         name = app['Name'] or app['Auto Name']
         if app['Summary'] and name:
             if app['Summary'].lower() == name.lower():
                 warn("Summary '%s' is just the app's name" % app['Summary'])
 
-        if app['Summary'] and app['Description'] and len(app['Description']) == 1:
-            if app['Summary'].lower() == app['Description'][0].lower():
+        desc = app['Description']
+        if app['Summary'] and desc and len(desc) == 1:
+            if app['Summary'].lower() == desc[0].lower():
                 warn("Description '%s' is just the app's summary" % app['Summary'])
 
         # Description size limit
-        desc_chars = sum(len(l) for l in app['Description'])
-        if desc_chars > config['char_limits']['Description']:
+        desc_charcount = sum(len(l) for l in desc)
+        if desc_charcount > config['char_limits']['Description']:
             warn("Description of length %s is over the %i char limit" % (
-                desc_chars, config['char_limits']['Description']))
+                desc_charcount, config['char_limits']['Description']))
+
+        if (not desc[0] or not desc[-1]
+                or any(not desc[l - 1] and not desc[l] for l in range(1, len(desc)))):
+            warn("Description has an extra empty line")
 
         # Regex checks in all kinds of fields
         for f in regex_warnings:
@@ -228,13 +213,6 @@ def main():
                         if m.match(l):
                             warn("%s at line '%s': %s" % (f, l, r))
 
-        # Regex pedantic checks in all kinds of fields
-        if options.pedantic:
-            for f in regex_pedantic:
-                for m, r in regex_pedantic[f]:
-                    if m.match(app[f]):
-                        warn("%s '%s': %s" % (f, app[f], r))
-
         # Build warnings
         for build in app['builds']:
             if build['disable']:
@@ -248,13 +226,6 @@ def main():
                     if ref.startswith(s):
                         warn("Branch '%s' used as commit in srclib '%s'" % (
                             s, srclib))
-            for s in ['git clone', 'git svn clone', 'svn checkout', 'svn co', 'hg clone']:
-                for flag in ['init', 'prebuild', 'build']:
-                    if not build[flag]:
-                        continue
-                    if s in build[flag]:
-                        # TODO: This should not be pedantic!
-                        pwarn("'%s' used in %s '%s'" % (s, flag, build[flag]))
 
         if not curid:
             print
@@ -262,5 +233,7 @@ def main():
     logging.info("Found a total of %i warnings in %i apps out of %i total." % (
         count['warn'], count['app'], count['app_total']))
 
+    sys.exit(1 if count['warn'] > 0 else 0)
+
 if __name__ == "__main__":
     main()
