@@ -71,7 +71,8 @@ def main():
         sys.exit(1)
 
     # Get all metadata-defined apps...
-    metaapps = [a for a in metadata.read_metadata().itervalues() if not a['Disabled']]
+    allmetaapps = [a for a in metadata.read_metadata().itervalues()]
+    metaapps = [a for a in allmetaapps if not a['Disabled']]
 
     statsdir = 'stats'
     logsdir = os.path.join(statsdir, 'logs')
@@ -225,8 +226,7 @@ def main():
             rtype = common.getsrclibvcs(app['Repo'])
         repotypes[rtype] += 1
     f = open('stats/repotypes.txt', 'w')
-    for rtype in repotypes:
-        count = repotypes[rtype]
+    for rtype, count in repotypes.most_common():
         f.write(rtype + ' ' + str(count) + '\n')
     f.close()
 
@@ -241,8 +241,7 @@ def main():
             checkmode = checkmode[:4]
         ucms[checkmode] += 1
     f = open('stats/update_check_modes.txt', 'w')
-    for checkmode in ucms:
-        count = ucms[checkmode]
+    for checkmode, count in ucms.most_common():
         f.write(checkmode + ' ' + str(count) + '\n')
     f.close()
 
@@ -252,8 +251,7 @@ def main():
         for category in app['Categories']:
             ctgs[category] += 1
     f = open('stats/categories.txt', 'w')
-    for category in ctgs:
-        count = ctgs[category]
+    for category, count in ctgs.most_common():
         f.write(category + ' ' + str(count) + '\n')
     f.close()
 
@@ -266,8 +264,7 @@ def main():
         for antifeature in antifeatures:
             afs[antifeature] += 1
     f = open('stats/antifeatures.txt', 'w')
-    for antifeature in afs:
-        count = afs[antifeature]
+    for antifeature, count in afs.most_common():
         f.write(antifeature + ' ' + str(count) + '\n')
     f.close()
 
@@ -278,17 +275,24 @@ def main():
         license = app['License']
         licenses[license] += 1
     f = open('stats/licenses.txt', 'w')
-    for license in licenses:
-        count = licenses[license]
+    for license, count in licenses.most_common():
         f.write(license + ' ' + str(count) + '\n')
     f.close()
 
+    # Write list of disabled apps...
+    logging.info("Processing disabled apps...")
+    disabled = [a['id'] for a in allmetaapps if a['Disabled']]
+    f = open('stats/disabled_apps.txt', 'w')
+    for appid in sorted(disabled):
+        f.write(appid + '\n')
+    f.close()
+
     # Write list of latest apps added to the repo...
     logging.info("Processing latest apps...")
     latest = knownapks.getlatest(10)
     f = open('stats/latestapps.txt', 'w')
-    for app in latest:
-        f.write(app + '\n')
+    for appid in latest:
+        f.write(appid + '\n')
     f.close()
 
     if unknownapks:
