@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 #
 # update.py - part of the FDroid server tools
-# Copyright (C) 2010-2013, Ciaran Gultnieks, ciaran@ciarang.com
+# Copyright (C) 2010-2015, Ciaran Gultnieks, ciaran@ciarang.com
 # Copyright (C) 2013-2014 Daniel Martí <mvdan@mvdan.cc>
 #
 # This program is free software: you can redistribute it and/or modify
@@ -23,9 +23,11 @@ import os
 import shutil
 import glob
 import re
+import socket
 import zipfile
 import hashlib
 import pickle
+from datetime import datetime, timedelta
 from xml.dom.minidom import Document
 from optparse import OptionParser
 import time
@@ -33,6 +35,7 @@ from pyasn1.error import PyAsn1Error
 from pyasn1.codec.der import decoder, encoder
 from pyasn1_modules import rfc2315
 from hashlib import md5
+from binascii import hexlify, unhexlify
 
 from PIL import Image
 import logging
@@ -92,7 +95,7 @@ def update_wiki(apps, sortedids, apks):
         if app['AntiFeatures']:
             for af in app['AntiFeatures'].split(','):
                 wikidata += '{{AntiFeature|' + af + '}}\n'
-        wikidata += '{{App|id=%s|name=%s|added=%s|lastupdated=%s|source=%s|tracker=%s|web=%s|donate=%s|flattr=%s|bitcoin=%s|litecoin=%s|dogecoin=%s|license=%s|root=%s}}\n' % (
+        wikidata += '{{App|id=%s|name=%s|added=%s|lastupdated=%s|source=%s|tracker=%s|web=%s|changelog=%s|donate=%s|flattr=%s|bitcoin=%s|litecoin=%s|dogecoin=%s|license=%s|root=%s}}\n' % (
             appid,
             app['Name'],
             time.strftime('%Y-%m-%d', app['added']) if 'added' in app else '',
@@ -100,6 +103,7 @@ def update_wiki(apps, sortedids, apks):
             app['Source Code'],
             app['Issue Tracker'],
             app['Web Site'],
+            app['Changelog'],
             app['Donate'],
             app['FlattrID'],
             app['Bitcoin'],
@@ -265,7 +269,7 @@ def update_wiki(apps, sortedids, apks):
                     newpage = site.Pages[pagename]
                     newpage.save(text, summary='Auto-created')
                 except:
-                    logging.error("...FAILED to create page")
+                    logging.error("...FAILED to create page '{0}'".format(pagename))
 
     # Purge server cache to ensure counts are up to date
     site.pages['Repository Maintenance'].purge()
@@ -327,7 +331,8 @@ def resize_all_icons(repodirs):
                 resize_icon(iconpath, density)
 
 
-cert_path_regex = re.compile(r'^META-INF/.*\.RSA$')
+# A signature block file with a .DSA, .RSA, or .EC extension
+cert_path_regex = re.compile(r'^META-INF/.*\.(DSA|EC|RSA)$')
 
 
 def getsig(apkpath):
@@ -420,14 +425,30 @@ def scan_apks(apps, apkcache, repodir, knownapks):
             logging.critical("Spaces in filenames are not allowed.")
             sys.exit(1)
 
-        if apkfilename in apkcache:
-            logging.debug("Reading " + apkfilename + " from cache")
-            thisinfo = apkcache[apkfilename]
+        # Calculate the sha256...
+        sha = hashlib.sha256()
+        with open(apkfile, 'rb') as f:
+            while True:
+                t = f.read(16384)
+                if len(t) == 0:
+                    break
+                sha.update(t)
+            shasum = sha.hexdigest()
 
-        else:
+        usecache = False
+        if apkfilename in apkcache:
+            thisinfo = apkcache[apkfilename]
+            if thisinfo['sha256'] == shasum:
+                logging.debug("Reading " + apkfilename + " from cache")
+                usecache = True
+            else:
+                logging.debug("Ignoring stale cache data for " + apkfilename)
+
+        if not usecache:
             logging.debug("Processing " + apkfilename)
             thisinfo = {}
             thisinfo['apkname'] = apkfilename
+            thisinfo['sha256'] = shasum
             srcfilename = apkfilename[:-4] + "_src.tar.gz"
             if os.path.exists(os.path.join(repodir, srcfilename)):
                 thisinfo['srcname'] = srcfilename
@@ -513,17 +534,8 @@ def scan_apks(apps, apkcache, repodir, knownapks):
             if common.isApkDebuggable(apkfile, config):
                 logging.warn('{0} is set to android:debuggable="true"'.format(apkfile))
 
-            # Calculate the sha256...
-            sha = hashlib.sha256()
-            with open(apkfile, 'rb') as f:
-                while True:
-                    t = f.read(1024)
-                    if len(t) == 0:
-                        break
-                    sha.update(t)
-                thisinfo['sha256'] = sha.hexdigest()
-
             # Get the signature (or md5 of, to be precise)...
+            logging.debug('Getting signature of {0}'.format(apkfile))
             thisinfo['sig'] = getsig(os.path.join(os.getcwd(), apkfile))
             if not thisinfo['sig']:
                 logging.critical("Failed to get apk signature")
@@ -531,6 +543,19 @@ def scan_apks(apps, apkcache, repodir, knownapks):
 
             apk = zipfile.ZipFile(apkfile, 'r')
 
+            # if an APK has files newer than the system time, suggest updating
+            # the system clock.  This is useful for offline systems, used for
+            # signing, which do not have another source of clock sync info. It
+            # has to be more than 24 hours newer because ZIP/APK files do not
+            # store timezone info
+            info = apk.getinfo('AndroidManifest.xml')
+            dt_obj = datetime(*info.date_time)
+            checkdt = dt_obj - timedelta(1)
+            if datetime.today() < checkdt:
+                logging.warn('System clock is older than manifest in: '
+                             + apkfilename + '\nSet clock to that time using:\n'
+                             + 'sudo date -s "' + str(dt_obj) + '"')
+
             iconfilename = "%s.%s.png" % (
                 thisinfo['id'],
                 thisinfo['versioncode'])
@@ -656,6 +681,36 @@ def scan_apks(apps, apkcache, repodir, knownapks):
 repo_pubkey_fingerprint = None
 
 
+# Generate a certificate fingerprint the same way keytool does it
+# (but with slightly different formatting)
+def cert_fingerprint(data):
+    digest = hashlib.sha256(data).digest()
+    ret = []
+    ret.append(' '.join("%02X" % ord(b) for b in digest))
+    return " ".join(ret)
+
+
+def extract_pubkey():
+    global repo_pubkey_fingerprint
+    if 'repo_pubkey' in config:
+        pubkey = unhexlify(config['repo_pubkey'])
+    else:
+        p = FDroidPopen(['keytool', '-exportcert',
+                         '-alias', config['repo_keyalias'],
+                         '-keystore', config['keystore'],
+                         '-storepass:file', config['keystorepassfile']]
+                        + config['smartcardoptions'], output=False)
+        if p.returncode != 0 or len(p.output) < 20:
+            msg = "Failed to get repo pubkey!"
+            if config['keystore'] == 'NONE':
+                msg += ' Is your crypto smartcard plugged in?'
+            logging.critical(msg)
+            sys.exit(1)
+        pubkey = p.output
+    repo_pubkey_fingerprint = cert_fingerprint(pubkey)
+    return hexlify(pubkey)
+
+
 def make_index(apps, sortedids, apks, repodir, archive, categories):
     """Make a repo index.
 
@@ -674,6 +729,11 @@ def make_index(apps, sortedids, apks, repodir, archive, categories):
         el.appendChild(doc.createTextNode(value))
         parent.appendChild(el)
 
+    def addElementNonEmpty(name, value, doc, parent):
+        if not value:
+            return
+        addElement(name, value, doc, parent)
+
     def addElementCDATA(name, value, doc, parent):
         el = doc.createElement(name)
         el.appendChild(doc.createCDATASection(value))
@@ -700,37 +760,32 @@ def make_index(apps, sortedids, apks, repodir, archive, categories):
         repoel.setAttribute("url", config['repo_url'])
         addElement('description', config['repo_description'], doc, repoel)
 
-    repoel.setAttribute("version", "12")
+    repoel.setAttribute("version", "13")
     repoel.setAttribute("timestamp", str(int(time.time())))
 
-    if 'repo_keyalias' in config:
-
-        # Generate a certificate fingerprint the same way keytool does it
-        # (but with slightly different formatting)
-        def cert_fingerprint(data):
-            digest = hashlib.sha256(data).digest()
-            ret = []
-            ret.append(' '.join("%02X" % ord(b) for b in digest))
-            return " ".join(ret)
-
-        def extract_pubkey():
-            p = FDroidPopen(['keytool', '-exportcert',
-                             '-alias', config['repo_keyalias'],
-                             '-keystore', config['keystore'],
-                             '-storepass:file', config['keystorepassfile']]
-                            + config['smartcardoptions'], output=False)
-            if p.returncode != 0:
-                msg = "Failed to get repo pubkey!"
-                if config['keystore'] == 'NONE':
-                    msg += ' Is your crypto smartcard plugged in?'
-                logging.critical(msg)
-                sys.exit(1)
-            global repo_pubkey_fingerprint
-            repo_pubkey_fingerprint = cert_fingerprint(p.output)
-            return "".join("%02x" % ord(b) for b in p.output)
-
-        repoel.setAttribute("pubkey", extract_pubkey())
+    nosigningkey = False
+    if not options.nosign:
+        if 'repo_keyalias' not in config:
+            nosigningkey = True
+            logging.critical("'repo_keyalias' not found in config.py!")
+        if 'keystore' not in config:
+            nosigningkey = True
+            logging.critical("'keystore' not found in config.py!")
+        if 'keystorepass' not in config and 'keystorepassfile' not in config:
+            nosigningkey = True
+            logging.critical("'keystorepass' not found in config.py!")
+        if 'keypass' not in config and 'keypassfile' not in config:
+            nosigningkey = True
+            logging.critical("'keypass' not found in config.py!")
+        if not os.path.exists(config['keystore']):
+            nosigningkey = True
+            logging.critical("'" + config['keystore'] + "' does not exist!")
+        if nosigningkey:
+            logging.warning("`fdroid update` requires a signing key, you can create one using:")
+            logging.warning("\tfdroid update --create-key")
+            sys.exit(1)
 
+    repoel.setAttribute("pubkey", extract_pubkey())
     root.appendChild(repoel)
 
     for appid in sortedids:
@@ -771,7 +826,7 @@ def make_index(apps, sortedids, apks, repodir, archive, categories):
                    metadata.description_html(app['Description'], linkres),
                    doc, apel)
         addElement('license', app['License'], doc, apel)
-        if 'Categories' in app:
+        if 'Categories' in app and app['Categories']:
             addElement('categories', ','.join(app["Categories"]), doc, apel)
             # We put the first (primary) category in LAST, which will have
             # the desired effect of making clients that only understand one
@@ -780,16 +835,12 @@ def make_index(apps, sortedids, apks, repodir, archive, categories):
         addElement('web', app['Web Site'], doc, apel)
         addElement('source', app['Source Code'], doc, apel)
         addElement('tracker', app['Issue Tracker'], doc, apel)
-        if app['Donate']:
-            addElement('donate', app['Donate'], doc, apel)
-        if app['Bitcoin']:
-            addElement('bitcoin', app['Bitcoin'], doc, apel)
-        if app['Litecoin']:
-            addElement('litecoin', app['Litecoin'], doc, apel)
-        if app['Dogecoin']:
-            addElement('dogecoin', app['Dogecoin'], doc, apel)
-        if app['FlattrID']:
-            addElement('flattr', app['FlattrID'], doc, apel)
+        addElementNonEmpty('changelog', app['Changelog'], doc, apel)
+        addElementNonEmpty('donate', app['Donate'], doc, apel)
+        addElementNonEmpty('bitcoin', app['Bitcoin'], doc, apel)
+        addElementNonEmpty('litecoin', app['Litecoin'], doc, apel)
+        addElementNonEmpty('dogecoin', app['Dogecoin'], doc, apel)
+        addElementNonEmpty('flattr', app['FlattrID'], doc, apel)
 
         # These elements actually refer to the current version (i.e. which
         # one is recommended. They are historically mis-named, and need
@@ -799,17 +850,11 @@ def make_index(apps, sortedids, apks, repodir, archive, categories):
 
         if app['AntiFeatures']:
             af = app['AntiFeatures'].split(',')
-            # TODO: Temporarily not including UpstreamNonFree in the index,
-            # because current F-Droid clients do not understand it, and also
-            # look ugly when they encounter an unknown antifeature. This
-            # filtering can be removed in time...
-            if 'UpstreamNonFree' in af:
-                af.remove('UpstreamNonFree')
             if af:
-                addElement('antifeatures', ','.join(af), doc, apel)
+                addElementNonEmpty('antifeatures', ','.join(af), doc, apel)
         if app['Provides']:
             pv = app['Provides'].split(',')
-            addElement('provides', ','.join(pv), doc, apel)
+            addElementNonEmpty('provides', ','.join(pv), doc, apel)
         if app['Requires Root']:
             addElement('requirements', 'root', doc, apel)
 
@@ -854,16 +899,10 @@ def make_index(apps, sortedids, apks, repodir, archive, categories):
                 addElement('maxsdkver', str(apk['maxsdkversion']), doc, apkel)
             if 'added' in apk:
                 addElement('added', time.strftime('%Y-%m-%d', apk['added']), doc, apkel)
-            if app['Requires Root']:
-                if 'ACCESS_SUPERUSER' not in apk['permissions']:
-                    apk['permissions'].add('ACCESS_SUPERUSER')
-
-            if len(apk['permissions']) > 0:
-                addElement('permissions', ','.join(apk['permissions']), doc, apkel)
-            if 'nativecode' in apk and len(apk['nativecode']) > 0:
+            addElementNonEmpty('permissions', ','.join(apk['permissions']), doc, apkel)
+            if 'nativecode' in apk:
                 addElement('nativecode', ','.join(apk['nativecode']), doc, apkel)
-            if len(apk['features']) > 0:
-                addElement('features', ','.join(apk['features']), doc, apkel)
+            addElementNonEmpty('features', ','.join(apk['features']), doc, apkel)
 
         if current_version_file is not None \
                 and config['make_current_version_link'] \
@@ -894,29 +933,38 @@ def make_index(apps, sortedids, apks, repodir, archive, categories):
 
     if 'repo_keyalias' in config:
 
-        logging.info("Creating signed index with this key (SHA256):")
-        logging.info("%s" % repo_pubkey_fingerprint)
+        if options.nosign:
+            logging.info("Creating unsigned index in preparation for signing")
+        else:
+            logging.info("Creating signed index with this key (SHA256):")
+            logging.info("%s" % repo_pubkey_fingerprint)
 
         # Create a jar of the index...
-        p = FDroidPopen(['jar', 'cf', 'index.jar', 'index.xml'], cwd=repodir)
+        jar_output = 'index_unsigned.jar' if options.nosign else 'index.jar'
+        p = FDroidPopen(['jar', 'cf', jar_output, 'index.xml'], cwd=repodir)
         if p.returncode != 0:
-            logging.critical("Failed to create jar file")
+            logging.critical("Failed to create {0}".format(jar_output))
             sys.exit(1)
 
         # Sign the index...
-        args = ['jarsigner', '-keystore', config['keystore'],
-                '-storepass:file', config['keystorepassfile'],
-                '-digestalg', 'SHA1', '-sigalg', 'MD5withRSA',
-                os.path.join(repodir, 'index.jar'), config['repo_keyalias']]
-        if config['keystore'] == 'NONE':
-            args += config['smartcardoptions']
-        else:  # smardcards never use -keypass
-            args += ['-keypass:file', config['keypassfile']]
-        p = FDroidPopen(args)
-        # TODO keypass should be sent via stdin
-        if p.returncode != 0:
-            logging.critical("Failed to sign index")
-            sys.exit(1)
+        signed = os.path.join(repodir, 'index.jar')
+        if options.nosign:
+            # Remove old signed index if not signing
+            if os.path.exists(signed):
+                os.remove(signed)
+        else:
+            args = ['jarsigner', '-keystore', config['keystore'],
+                    '-storepass:file', config['keystorepassfile'],
+                    '-digestalg', 'SHA1', '-sigalg', 'MD5withRSA',
+                    signed, config['repo_keyalias']]
+            if config['keystore'] == 'NONE':
+                args += config['smartcardoptions']
+            else:  # smardcards never use -keypass
+                args += ['-keypass:file', config['keypassfile']]
+            p = FDroidPopen(args)
+            if p.returncode != 0:
+                logging.critical("Failed to sign index")
+                sys.exit(1)
 
     # Copy the repo icon into the repo directory...
     icon_dir = os.path.join(repodir, 'icons')
@@ -978,6 +1026,8 @@ def main():
 
     # Parse command line...
     parser = OptionParser()
+    parser.add_option("--create-key", action="store_true", default=False,
+                      help="Create a repo signing key in a keystore")
     parser.add_option("-c", "--create-metadata", action="store_true", default=False,
                       help="Create skeleton metadata files that are missing")
     parser.add_option("--delete-unknown", action="store_true", default=False,
@@ -1001,6 +1051,8 @@ def main():
                       help="Produce human-readable index.xml")
     parser.add_option("--clean", action="store_true", default=False,
                       help="Clean update - don't uses caches, reprocess all apks")
+    parser.add_option("--nosign", action="store_true", default=False,
+                      help="When configured for signed indexes, create only unsigned indexes at this stage")
     (options, args) = parser.parse_args()
 
     config = common.read_config(options)
@@ -1022,6 +1074,32 @@ def main():
                 logging.critical(k + ' "' + config[k] + '" does not exist! Correct it in config.py.')
                 sys.exit(1)
 
+    # if the user asks to create a keystore, do it now, reusing whatever it can
+    if options.create_key:
+        if os.path.exists(config['keystore']):
+            logging.critical("Cowardily refusing to overwrite existing signing key setup!")
+            logging.critical("\t'" + config['keystore'] + "'")
+            sys.exit(1)
+
+        if 'repo_keyalias' not in config:
+            config['repo_keyalias'] = socket.getfqdn()
+            common.write_to_config(config, 'repo_keyalias', config['repo_keyalias'])
+        if 'keydname' not in config:
+            config['keydname'] = 'CN=' + config['repo_keyalias'] + ', OU=F-Droid'
+            common.write_to_config(config, 'keydname', config['keydname'])
+        if 'keystore' not in config:
+            config['keystore'] = common.default_config.keystore
+            common.write_to_config(config, 'keystore', config['keystore'])
+
+        password = common.genpassword()
+        if 'keystorepass' not in config:
+            config['keystorepass'] = password
+            common.write_to_config(config, 'keystorepass', config['keystorepass'])
+        if 'keypass' not in config:
+            config['keypass'] = password
+            common.write_to_config(config, 'keypass', config['keypass'])
+        common.genkeystore(config)
+
     # Get all apps...
     apps = metadata.read_metadata()
 
@@ -1064,6 +1142,7 @@ def main():
                 f.write("Web Site:\n")
                 f.write("Source Code:\n")
                 f.write("Issue Tracker:\n")
+                f.write("Changelog:\n")
                 f.write("Summary:" + apk['name'] + "\n")
                 f.write("Description:\n")
                 f.write(apk['name'] + "\n")
