@@ -171,7 +171,7 @@ class FDroid
 			$out.=$this->get_app($query_vars);
 		} else {
 			$out.='<form name="searchform" action="" method="get">';
-			$out.='<p><input name="fdfilter" type="text" value="'.$query_vars['fdfilter'].'" size="30"> ';
+			$out.='<p><input name="fdfilter" type="text" value="'.esc_attr($query_vars['fdfilter']).'" size="30"> ';
 			$out.='<input type="hidden" name="fdpage" value="1">';
 			$out.='<input type="submit" value="Search"></p>';
 			$out.=$this->makeformdata($query_vars);
@@ -221,8 +221,10 @@ class FDroid
 		}
 	}
 	function androidversion($sdkLevel) {
-		if ($sdkLevel < 1) return null;
 		switch ($sdkLevel) {
+			case 22: return "5.1";
+			case 21: return "5.0";
+			case 20: return "4.4W";
 			case 19: return "4.4";
 			case 18: return "4.3";
 			case 17: return "4.2";
@@ -283,6 +285,9 @@ class FDroid
 					case "tracker":
 						$issues=$el;
 						break;
+					case "changelog":
+						$changelog=$el;
+						break;
 					case "donate":
 						$donate=$el;
 						break;
@@ -397,6 +402,8 @@ class FDroid
 					$out.='<b>Issue Tracker:</b> <a href="'.$issues.'">'.$issues.'</a><br />';
 				if(strlen($source)>0)
 					$out.='<b>Source Code:</b> <a href="'.$source.'">'.$source.'</a><br />';
+				if(strlen($changelog)>0)
+					$out.='<b>Changelog:</b> <a href="'.$changelog.'">'.$changelog.'</a><br />';
 				if(isset($donate) && strlen($donate)>0)
 					$out.='<b>Donate:</b> <a href="'.$donate.'">'.$donate.'</a><br />';
 				if(isset($flattr) && strlen($flattr)>0)
@@ -689,7 +696,7 @@ class FDroid
 				$out.='</form>'."\n";
 			}
 			else {
-				$out.='Applications matching "'.$query_vars['fdfilter'].'"';
+				$out.='Applications matching "'.esc_attr($query_vars['fdfilter']).'"';
 			}
 			$out.="</div>";
 
@@ -748,7 +755,7 @@ class FDroid
 		$out.='<input type="hidden" name="page_id" value="'.(int)get_query_var('page_id').'">';
 		foreach($query_vars as $name => $value) {
 			if($value !== null && $name != 'fdfilter' && $name != 'fdpage')
-				$out.='<input type="hidden" name="'.$name.'" value="'.sanitize_text_field($value).'">';
+				$out.='<input type="hidden" name="'.esc_attr($name).'" value="'.esc_attr($value).'">';
 		}
 
 		return $out;
